#include <fxlib.h>
#include "blockdude.h"
#include "level.h"
#include "tileset.h"
#include "MonochromeLib.h"

#define MIN(x,y) (x<y?x:y)
#define MAX(x,y) (x>y?x:y)

int play(int n) {
	Level *level = level_load(n);
	Tileset *perso = tileset_load(PERSO);
	Tileset *wall = tileset_load(WALL);
	
	int cameraX = MIN(MAX(0, level->posX-7), level->lvWidth-16), cameraY = MIN(MAX(0, level->posY-4), level->lvHeight-8);
	int shiftX, shiftY;
	int block = 0, fall;
	int key, i, j;
	
	while(1) {
		ML_clear_vram();
		for(j = 0 ; j < MIN(8, level->lvHeight-cameraY) ; j++) {
			 for(i = 0 ; i < MIN(16, level->lvWidth-cameraX) ; i++) {
				switch(level->lv[level->lvWidth*(cameraY+j)+(cameraX+i)]) {
					case 1 :
						ML_rectangle(8*i+1, 8*j, 8*i+6, 8*j+7, 1, ML_BLACK, ML_WHITE);
						ML_pixel(8*i+5, 8*j+4, ML_BLACK);
					break;
					case 2 : 
						ML_bmp_8_or(wall->tile[0], 8*i, 8*j);
					break;
					case 3 :
						ML_rectangle(8*i, 8*j, 8*i+7, 8*j+7, 1, ML_BLACK, ML_WHITE);
				}
			}
		}
		ML_bmp_or(perso->tile[(level->dir+1)/2], 8*(level->posX-cameraX)+1, 8*(level->posY-cameraY), 6, 8);
		if(block) ML_rectangle(8*(level->posX-cameraX), 8*(level->posY-cameraY-1), 8*(level->posX-cameraX)+7, 8*(level->posY-cameraY-1)+7, 1, ML_BLACK, ML_WHITE);
		ML_display_vram();
		
		GetKey(&key);
		if(key == KEY_CTRL_RIGHT) {
			if(level->lv[level->lvWidth*level->posY+level->posX+1] <= 1 ) {
				if(block && level->lv[level->lvWidth*(level->posY-1)+level->posX+1] != 0) {
					block = 2;
					fall = level->lvWidth*(level->posY-1)+level->posX;
				}
				level->posX++;
				if(level->posX-cameraX > 11 && level->lvWidth-cameraX > 16) cameraX++;
				while(level->lv[level->lvWidth*(level->posY+1)+level->posX] <= 1) {
					level->posY++;
					if(level->posY-cameraY > 5 && level->lvWidth-cameraY > 8) cameraY++;
				}
			}
			level->dir = 1;
		}
		if(key == KEY_CTRL_LEFT) {
			if(level->lv[level->lvWidth*level->posY+level->posX-1] <= 1) {
				if(block && level->lv[level->lvWidth*(level->posY-1)+level->posX-1] != 0) {
					block = 2;
					fall = level->lvWidth*(level->posY-1)+level->posX;
				}
				level->posX--;
				if(level->posX-cameraX < 5 && cameraX > 0) cameraX--;
				while(level->lv[level->lvWidth*(level->posY+1)+level->posX] <= 1 ) {
					level->posY++;
					if(level->posY-cameraY > 5 && level->lvWidth-cameraY > 8) cameraY++;
				}
			}
			level->dir = -1;
		}
		if(key == KEY_CTRL_UP && level->lv[level->lvWidth*(level->posY-1)+level->posX] == 0 && level->lv[level->lvWidth*(level->posY-1)+level->posX+level->dir] <= 1 && level->lv[level->lvWidth*(level->posY)+level->posX+level->dir] > 1) {
			if(block && level->lv[level->lvWidth*(level->posY-2)+level->posX+level->dir] != 0) {
				block = 2;
				fall = level->lvWidth*(level->posY-1)+level->posX;
			}
			level->posX += level->dir;
			if(level->posX-cameraX > 11 && level->lvWidth-cameraX > 16) cameraX++;
			if(level->posX-cameraX < 5 && cameraX > 0) cameraX--;
			level->posY--;
			if(level->posY-cameraY < 3 && cameraY > 0) cameraY--;
		}
		if(key == KEY_CTRL_DOWN && block && level->lv[level->lvWidth*(level->posY-1)+level->posX+level->dir] == 0 && level->lv[level->lvWidth*level->posY+level->posX+level->dir] != 1) {
			block = 2;
			fall = level->lvWidth*(level->posY-1)+level->posX+level->dir;
		}
		if(key == KEY_CTRL_DOWN && !block && level->lv[level->lvWidth*(level->posY-1)+level->posX] == 0 && level->lv[level->lvWidth*(level->posY-1)+level->posX+level->dir] == 0 && level->lv[level->lvWidth*level->posY+level->posX+level->dir] == 3) {
			level->lv[level->lvWidth*level->posY+level->posX+level->dir] = 0;
			block = 1;
		}
		if(block == 2) {
			while(level->lv[fall+level->lvWidth] == 0) fall += level->lvWidth;
			level->lv[fall] = 3;
			block = 0;
		}
		if(key == KEY_CTRL_SHIFT) {
			shiftX = cameraX;
			shiftY = cameraY;
			while(IsKeyDown(KEY_CTRL_SHIFT)) {
				ML_clear_vram();
				for(j = 0 ; j < MIN(8, level->lvHeight-shiftY) ; j++) {
					 for(i = 0 ; i < MIN(16, level->lvWidth-shiftX) ; i++) {
						switch(level->lv[level->lvWidth*(shiftY+j)+(shiftX+i)]) {
							case 1 :
								ML_rectangle(8*i+1, 8*j, 8*i+6, 8*j+7, 1, ML_BLACK, ML_WHITE);
								ML_pixel(8*i+5, 8*j+4, ML_BLACK);
							break;
							case 2 : 
								ML_bmp_8_or(wall->tile[0], 8*i, 8*j);
							break;
							case 3 :
								ML_rectangle(8*i, 8*j, 8*i+7, 8*j+7, 1, ML_BLACK, ML_WHITE);
						}
					}
				}
				if(level->posX-shiftX >= 0 && level->posX-shiftX < 16 && level->posY-shiftY >= 0 && level->posY-shiftY < 8) ML_bmp_or(perso->tile[(level->dir+1)/2], 8*(level->posX-shiftX)+1, 8*(level->posY-shiftY), 6, 8);
				if(block && level->posX-shiftX >= 0 && level->posX-shiftX < 16 && level->posY-shiftY-1 >= 0 && level->posY-shiftY-1 < 8) ML_rectangle(8*(level->posX-shiftX), 8*(level->posY-shiftY-1), 8*(level->posX-shiftX)+7, 8*(level->posY-shiftY-1)+7, 1, ML_BLACK, ML_WHITE);
				ML_display_vram();
				
				Sleep(50);
				if(IsKeyDown(KEY_CTRL_RIGHT) && level->lvWidth-shiftX > 16) shiftX++;
				if(IsKeyDown(KEY_CTRL_LEFT) && shiftX > 0) shiftX--;
				if(IsKeyDown(KEY_CTRL_DOWN) && level->lvHeight-shiftY > 8) shiftY++;
				if(IsKeyDown(KEY_CTRL_UP) && shiftY > 0) shiftY--;
			}
		}
		if(key == KEY_CHAR_6) {
			level_free(level);
			tileset_free(perso);
			tileset_free(wall);
			return 2;
		}
		if(key == KEY_CTRL_EXIT) {
			level_free(level);
			tileset_free(perso);
			tileset_free(wall);
			return 0;
		}
		if(level->lv[level->lvWidth*level->posY+level->posX] == 1) {
			level_free(level);
			tileset_free(perso);
			tileset_free(wall);
			return 1;
		}
	}
}